@@ -12,10 +12,10 @@ for i in xrange(1000000):
     pipe.execute()
 
     pipe = r.pipeline(transaction=False)
-    pipe.set("foo-{0}".format(d, d))
-    pipe.incrby("foo-{0}".format(d, 1))
-    pipe.set("bar-{0}".format(d, d))
-    pipe.incrby("bar-{0}".format(d, 1))
-    pipe.set("bazz-{0}".format(d, d))
-    pipe.incrby("bazz-{0}".format(d, 1))
+    pipe.set("foo-{0}".format(d), d)
+    pipe.incrby("foo-{0}".format(d), 1)
+    pipe.set("bar-{0}".format(d), d)
+    pipe.incrby("bar-{0}".format(d), 1)
+    pipe.set("bazz-{0}".format(d), d)
+    pipe.incrby("bazz-{0}".format(d), 1)
     pipe.execute()
