@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+from __future__ import unicode_literals
 
 # python std lib
 import datetime
@@ -7,10 +8,15 @@ import string
 import time
 
 # rediscluster imports
-from .connection import ClusterConnectionPool, ClusterReadOnlyConnectionPool
+from .connection import (
+    ClusterConnectionPool,
+    ClusterReadOnlyConnectionPool,
+    ClusterWithReadReplicasConnectionPool,
+    SSLClusterConnection,
+)
 from .exceptions import (
     RedisClusterException, AskError, MovedError, ClusterDownError,
-    ClusterError, TryAgainError,
+    ClusterError, TryAgainError
 )
 from .pubsub import ClusterPubSub
 from .utils import (
@@ -23,18 +29,21 @@ from .utils import (
     clusterdown_wrapper,
     parse_cluster_slots,
     parse_cluster_nodes,
+    parse_pubsub_channels,
+    parse_pubsub_numsub,
+    parse_pubsub_numpat,
 )
 # 3rd party imports
-from redis import StrictRedis
+from redis import Redis
 from redis.client import list_or_args, parse_info
 from redis.connection import Token
-from redis._compat import iteritems, basestring, b, izip, nativestr, long
+from redis._compat import iteritems, basestring, izip, nativestr, long
 from redis.exceptions import RedisError, ResponseError, TimeoutError, DataError, ConnectionError, BusyLoadingError
 
 
-class StrictRedisCluster(StrictRedis):
+class RedisCluster(Redis):
     """
-    If a command is implemented over the one in StrictRedis then it requires some changes compared to
+    If a command is implemented over the one in Redis then it requires some changes compared to
     the regular implementation of the method.
     """
     RedisClusterRequestTTL = 16
@@ -50,26 +59,47 @@ class StrictRedisCluster(StrictRedis):
             "ECHO", "CONFIG GET", "CONFIG SET", "SLOWLOG GET", "CLIENT KILL", "INFO",
             "BGREWRITEAOF", "BGSAVE", "CLIENT LIST", "CLIENT GETNAME", "CONFIG RESETSTAT",
             "CONFIG REWRITE", "DBSIZE", "LASTSAVE", "PING", "SAVE", "SLOWLOG LEN", "SLOWLOG RESET",
-            "TIME", "KEYS", "CLUSTER INFO",
+            "TIME", "KEYS", "CLUSTER INFO", "PUBSUB CHANNELS",
+            "PUBSUB NUMSUB", "PUBSUB NUMPAT", "CLIENT ID",
         ], 'all-nodes'),
         string_keys_to_dict([
             "FLUSHALL", "FLUSHDB", "SCRIPT LOAD", "SCRIPT FLUSH", "SCRIPT EXISTS", "SCAN",
         ], 'all-masters'),
         string_keys_to_dict([
-            "RANDOMKEY", "CLUSTER NODES", 'CLUSTER SLOTS',
+            "RANDOMKEY", "CLUSTER NODES", "CLUSTER SLOTS",
         ], 'random'),
         string_keys_to_dict([
             "CLUSTER COUNTKEYSINSLOT",
+            "CLUSTER GETKEYSINSLOT",
         ], 'slot-id'),
     )
 
+    # Not complete, but covers the major ones
+    # https://redis.io/commands
+    READ_COMMANDS = [
+        "BITPOS", "BITCOUNT",
+        "EXISTS",
+        "GEOHASH", "GEOPOS", "GEODIST", "GEORADIUS", "GEORADIUSBYMEMBER",
+        "GET", "GETBIT", "GETRANGE",
+        "HEXISTS", "HGET", "HGETALL", "HKEYS", "HLEN", "HMGET", "HSTRLEN", "HVALS",
+        "KEYS",
+        "LINDEX", "LLEN", "LRANGE",
+        "MGET",
+        "PTTL",
+        "RANDOMKEY",
+        "SCARD", "SDIFF", "SINTER", "SISMEMBER", "SMEMBERS", "SRANDMEMBER",
+        "STRLEN", "SUNION",
+        "TTL",
+        "ZCARD", "ZCOUNT", "ZRANGE", "ZSCORE"
+    ]
+
     RESULT_CALLBACKS = dict_merge(
         string_keys_to_dict([
             "ECHO", "CONFIG GET", "CONFIG SET", "SLOWLOG GET", "CLIENT KILL", "INFO",
             "BGREWRITEAOF", "BGSAVE", "CLIENT LIST", "CLIENT GETNAME", "CONFIG RESETSTAT",
             "CONFIG REWRITE", "DBSIZE", "LASTSAVE", "PING", "SAVE", "SLOWLOG LEN", "SLOWLOG RESET",
             "TIME", "SCAN", "CLUSTER INFO", 'CLUSTER ADDSLOTS', 'CLUSTER COUNT-FAILURE-REPORTS',
-            'CLUSTER DELSLOTS', 'CLUSTER FAILOVER', 'CLUSTER FORGET', "FLUSHALL", "FLUSHDB",
+            'CLUSTER DELSLOTS', 'CLUSTER FAILOVER', 'CLUSTER FORGET', "FLUSHALL", "FLUSHDB", "CLIENT ID",
         ], lambda command, res: res),
         string_keys_to_dict([
             "SCRIPT LOAD",
@@ -86,6 +116,15 @@ class StrictRedisCluster(StrictRedis):
         string_keys_to_dict([
             "SSCAN", "HSCAN", "ZSCAN", "RANDOMKEY",
         ], first_key),
+        string_keys_to_dict([
+            "PUBSUB CHANNELS",
+        ], parse_pubsub_channels),
+        string_keys_to_dict([
+            "PUBSUB NUMSUB",
+        ], parse_pubsub_numsub),
+        string_keys_to_dict([
+            "PUBSUB NUMPAT",
+        ], parse_pubsub_numpat),
     )
 
     CLUSTER_COMMANDS_RESPONSE_CALLBACKS = {
@@ -95,7 +134,7 @@ class StrictRedisCluster(StrictRedis):
         'CLUSTER DELSLOTS': bool_ok,
         'CLUSTER FAILOVER': bool_ok,
         'CLUSTER FORGET': bool_ok,
-        'CLUSTER GETKEYSINSLOT': int,
+        'CLUSTER GETKEYSINSLOT': list,
         'CLUSTER INFO': parse_info,
         'CLUSTER KEYSLOT': int,
         'CLUSTER MEET': bool_ok,
@@ -112,8 +151,9 @@ class StrictRedisCluster(StrictRedis):
         'READWRITE': bool_ok,
     }
 
-    def __init__(self, host=None, port=None, startup_nodes=None, max_connections=32, max_connections_per_node=False, init_slot_cache=True,
-                 readonly_mode=False, reinitialize_steps=None, skip_full_coverage_check=False, **kwargs):
+    def __init__(self, host=None, port=None, startup_nodes=None, max_connections=None, max_connections_per_node=False, init_slot_cache=True,
+                 readonly_mode=False, reinitialize_steps=None, skip_full_coverage_check=False, nodemanager_follow_cluster=False,
+                 connection_class=None, read_from_replicas=False, **kwargs):
         """
         :startup_nodes:
             List of nodes that initial bootstrapping can be done from
@@ -128,17 +168,24 @@ class StrictRedisCluster(StrictRedis):
         :skip_full_coverage_check:
             Skips the check of cluster-require-full-coverage config, useful for clusters
             without the CONFIG command (like aws)
+        :nodemanager_follow_cluster:
+            The node manager will during initialization try the last set of nodes that
+            it was operating on. This will allow the client to drift along side the cluster
+            if the cluster nodes move around alot.
         :**kwargs:
-            Extra arguments that will be sent into StrictRedis instance when created
+            Extra arguments that will be sent into Redis instance when created
             (See Official redis-py doc for supported kwargs
             [https://github.com/andymccurdy/redis-py/blob/master/redis/client.py])
             Some kwargs is not supported and will raise RedisClusterException
             - db (Redis do not support database SELECT in cluster mode)
         """
-        # Tweaks to StrictRedis client arguments when running in cluster mode
+        # Tweaks to Redis client arguments when running in cluster mode
         if "db" in kwargs:
             raise RedisClusterException("Argument 'db' is not possible to use in cluster mode")
 
+        if kwargs.pop('ssl', False):  # Needs to be removed to avoid exception in redis Connection init
+            connection_class = SSLClusterConnection
+
         if "connection_pool" in kwargs:
             pool = kwargs.pop('connection_pool')
         else:
@@ -150,6 +197,8 @@ class StrictRedisCluster(StrictRedis):
 
             if readonly_mode:
                 connection_pool_cls = ClusterReadOnlyConnectionPool
+            elif read_from_replicas:
+                connection_pool_cls = ClusterWithReadReplicasConnectionPool
             else:
                 connection_pool_cls = ClusterConnectionPool
 
@@ -160,19 +209,22 @@ class StrictRedisCluster(StrictRedis):
                 reinitialize_steps=reinitialize_steps,
                 max_connections_per_node=max_connections_per_node,
                 skip_full_coverage_check=skip_full_coverage_check,
+                nodemanager_follow_cluster=nodemanager_follow_cluster,
+                connection_class=connection_class,
                 **kwargs
             )
 
-        super(StrictRedisCluster, self).__init__(connection_pool=pool, **kwargs)
+        super(RedisCluster, self).__init__(connection_pool=pool, **kwargs)
 
         self.refresh_table_asap = False
         self.nodes_flags = self.__class__.NODES_FLAGS.copy()
         self.result_callbacks = self.__class__.RESULT_CALLBACKS.copy()
         self.response_callbacks = self.__class__.RESPONSE_CALLBACKS.copy()
         self.response_callbacks = dict_merge(self.response_callbacks, self.CLUSTER_COMMANDS_RESPONSE_CALLBACKS)
+        self.read_from_replicas = read_from_replicas
 
     @classmethod
-    def from_url(cls, url, db=None, skip_full_coverage_check=False, **kwargs):
+    def from_url(cls, url, db=None, skip_full_coverage_check=False, readonly_mode=False, read_from_replicas=False, **kwargs):
         """
         Return a Redis client object configured from the given URL, which must
         use either `the ``redis://`` scheme
@@ -192,7 +244,14 @@ class StrictRedisCluster(StrictRedis):
         passed along to the ConnectionPool class's initializer. In the case
         of conflicting arguments, querystring arguments always win.
         """
-        connection_pool = ClusterConnectionPool.from_url(url, db=db, **kwargs)
+        if readonly_mode:
+            connection_pool_cls = ClusterReadOnlyConnectionPool
+        elif read_from_replicas:
+            connection_pool_cls = ClusterWithReadReplicasConnectionPool
+        else:
+            connection_pool_cls = ClusterConnectionPool
+
+        connection_pool = connection_pool_cls.from_url(url, db=db, skip_full_coverage_check=skip_full_coverage_check, **kwargs)
         return cls(connection_pool=connection_pool, skip_full_coverage_check=skip_full_coverage_check)
 
     def __repr__(self):
@@ -202,6 +261,10 @@ class StrictRedisCluster(StrictRedis):
         servers.sort()
         return "{0}<{1}>".format(type(self).__name__, ', '.join(servers))
 
+    def set_result_callback(self, command, callback):
+        "Set a custom Result Callback"
+        self.result_callbacks[command] = callback
+
     def pubsub(self, **kwargs):
         """
         """
@@ -220,7 +283,7 @@ class StrictRedisCluster(StrictRedis):
         if transaction:
             raise RedisClusterException("transaction is deprecated in cluster mode")
 
-        return StrictClusterPipeline(
+        return ClusterPipeline(
             connection_pool=self.connection_pool,
             startup_nodes=self.connection_pool.nodes.startup_nodes,
             result_callbacks=self.result_callbacks,
@@ -231,7 +294,7 @@ class StrictRedisCluster(StrictRedis):
         """
         Transaction is not implemented in cluster mode yet.
         """
-        raise RedisClusterException("method StrictRedisCluster.transaction() is not implemented")
+        raise RedisClusterException("method RedisCluster.transaction() is not implemented")
 
     def _determine_slot(self, *args):
         """
@@ -251,14 +314,18 @@ class StrictRedisCluster(StrictRedis):
 
         key = args[1]
 
+        # OBJECT command uses a special keyword as first positional argument
+        if command == 'OBJECT':
+            key = args[2]
+
         return self.connection_pool.nodes.keyslot(key)
 
-    def _merge_result(self, command, res):
+    def _merge_result(self, command, res, **kwargs):
         """
         `res` is a dict with the following structure Dict(NodeName, CommandResult)
         """
         if command in self.result_callbacks:
-            return self.result_callbacks[command](command, res)
+            return self.result_callbacks[command](command, res, **kwargs)
 
         # Default way to handle result
         return first_key(command, res)
@@ -292,17 +359,18 @@ class StrictRedisCluster(StrictRedis):
 
         command = args[0]
 
-        node = self.determine_node(*args, **kwargs)
-        if node:
-            return self._execute_command_on_nodes(node, *args, **kwargs)
-
         # If set externally we must update it before calling any commands
         if self.refresh_table_asap:
             self.connection_pool.nodes.initialize()
             self.refresh_table_asap = False
 
+        node = self.determine_node(*args, **kwargs)
+        if node:
+            return self._execute_command_on_nodes(node, *args, **kwargs)
+
         redirect_addr = None
         asking = False
+        is_read_replica = False
 
         try_random_node = False
         slot = self._determine_slot(*args)
@@ -322,7 +390,8 @@ class StrictRedisCluster(StrictRedis):
                     # MOVED
                     node = self.connection_pool.get_master_node_by_slot(slot)
                 else:
-                    node = self.connection_pool.get_node_by_slot(slot)
+                    node = self.connection_pool.get_node_by_slot(slot, self.read_from_replicas and (command in self.READ_COMMANDS))
+                    is_read_replica = node['server_type'] == 'slave'
                 r = self.connection_pool.get_connection_by_node(node)
 
             try:
@@ -330,6 +399,12 @@ class StrictRedisCluster(StrictRedis):
                     r.send_command('ASKING')
                     self.parse_response(r, "ASKING", **kwargs)
                     asking = False
+                if is_read_replica:
+                    # Ask read replica to accept reads (see https://redis.io/commands/readonly)
+                    # TODO: do we need to handle errors from this response?
+                    r.send_command('READONLY')
+                    self.parse_response(r, 'READONLY', **kwargs)
+                    is_read_replica = False
 
                 r.send_command(*args)
                 return self.parse_response(r, command, **kwargs)
@@ -387,10 +462,16 @@ class StrictRedisCluster(StrictRedis):
 
                 connection.send_command(*args)
                 res[node["name"]] = self.parse_response(connection, command, **kwargs)
+            except ClusterDownError as e:
+                self.connection_pool.disconnect()
+                self.connection_pool.reset()
+                self.refresh_table_asap = True
+
+                raise
             finally:
                 self.connection_pool.release(connection)
 
-        return self._merge_result(command, res)
+        return self._merge_result(command, res, **kwargs)
 
     ##########
     # Cluster management commands
@@ -534,6 +615,12 @@ class StrictRedisCluster(StrictRedis):
         """
         return self.execute_command('CLUSTER SAVECONFIG')
 
+    def cluster_get_keys_in_slot(self, slot, num_keys):
+        """
+        Returns the number of keys in the specefied cluster slot
+        """
+        return self.execute_command('CLUSTER GETKEYSINSLOT', slot, num_keys)
+
     def cluster_set_config_epoch(self, node_id, epoch):
         """
         Set the configuration epoch in a new node
@@ -630,7 +717,7 @@ class StrictRedisCluster(StrictRedis):
 
         Cluster impl:
             Itterate all keys and send GET for each key.
-            This will go alot slower than a normal mget call in StrictRedis.
+            This will go alot slower than a normal mget call in Redis.
 
             Operation is no longer atomic.
         """
@@ -678,17 +765,35 @@ class StrictRedisCluster(StrictRedis):
 
         return self.mset(**kwargs)
 
-    def rename(self, src, dst):
+    def rename(self, src, dst, replace=False):
         """
         Rename key ``src`` to ``dst``
 
         Cluster impl:
-            This operation is no longer atomic because each key must be querried
-            then set in separate calls because they maybe will change cluster node
+            If the src and dsst keys is in the same slot then send a plain RENAME
+            command to that node to do the rename inside the server.
+
+            If the keys is in crossslots then use the client side implementation
+            as fallback method. In this case this operation is no longer atomic as
+            the key is dumped and posted back to the server through the client.
         """
         if src == dst:
             raise ResponseError("source and destination objects are the same")
 
+        #
+        # Optimization where if both keys is in the same slot then we can use the
+        # plain upstream rename method.
+        #
+        src_slot = self.connection_pool.nodes.keyslot(src)
+        dst_slot = self.connection_pool.nodes.keyslot(dst)
+
+        if src_slot == dst_slot:
+            return self.execute_command('RENAME', src, dst)
+
+        #
+        # To provide cross slot support we implement rename by doing the internal command
+        # redis server runs but in the client instead.
+        #
         data = self.dump(src)
 
         if data is None:
@@ -700,7 +805,7 @@ class StrictRedisCluster(StrictRedis):
             ttl = 0
 
         self.delete(dst)
-        self.restore(dst, ttl, data)
+        self.restore(dst, ttl, data, replace)
         self.delete(src)
 
         return True
@@ -711,7 +816,7 @@ class StrictRedisCluster(StrictRedis):
 
         Cluster impl:
             Iterate all keys and send DELETE for each key.
-            This will go a lot slower than a normal delete call in StrictRedis.
+            This will go a lot slower than a normal delete call in Redis.
 
             Operation is no longer atomic.
         """
@@ -736,6 +841,30 @@ class StrictRedisCluster(StrictRedis):
 
         return False
 
+    def pubsub_channels(self, pattern='*', aggregate=True):
+        """
+        Return a list of channels that have at least one subscriber.
+        Aggregate toggles merging of response.
+        """
+        return self.execute_command('PUBSUB CHANNELS', pattern, aggregate=aggregate)
+
+    def pubsub_numpat(self, aggregate=True):
+        """
+        Returns the number of subscriptions to patterns.
+        Aggregate toggles merging of response.
+        """
+        return self.execute_command('PUBSUB NUMPAT', aggregate=aggregate)
+
+    def pubsub_numsub(self, *args, **kwargs):
+        """
+        Return a list of (channel, number of subscribers) tuples
+        for each channel given in ``*args``.
+
+        ``aggregate`` keyword argument toggles merging of response.
+        """
+        options = {'aggregate': kwargs.get('aggregate', True)}
+        return self.execute_command('PUBSUB NUMSUB', *args, **options)
+
     ####
     # List commands
 
@@ -782,152 +911,6 @@ class StrictRedisCluster(StrictRedis):
 
         return None
 
-    def sort(self, name, start=None, num=None, by=None, get=None, desc=False, alpha=False, store=None, groups=None):
-        """Sort and return the list, set or sorted set at ``name``.
-
-        :start: and :num:
-            allow for paging through the sorted data
-
-        :by:
-            allows using an external key to weight and sort the items.
-            Use an "*" to indicate where in the key the item value is located
-
-        :get:
-            allows for returning items from external keys rather than the
-            sorted data itself.  Use an "*" to indicate where int he key
-            the item value is located
-
-        :desc:
-            allows for reversing the sort
-
-        :alpha:
-            allows for sorting lexicographically rather than numerically
-
-        :store:
-            allows for storing the result of the sort into the key `store`
-
-        ClusterImpl:
-            A full implementation of the server side sort mechanics because many of the
-            options work on multiple keys that can exist on multiple servers.
-        """
-        if (start is None and num is not None) or \
-           (start is not None and num is None):
-            raise RedisError("RedisError: ``start`` and ``num`` must both be specified")
-        try:
-            data_type = b(self.type(name))
-
-            if data_type == b("none"):
-                return []
-            elif data_type == b("set"):
-                data = list(self.smembers(name))[:]
-            elif data_type == b("list"):
-                data = self.lrange(name, 0, -1)
-            else:
-                raise RedisClusterException("Unable to sort data type : {0}".format(data_type))
-            if by is not None:
-                # _sort_using_by_arg mutates data so we don't
-                # need need a return value.
-                self._sort_using_by_arg(data, by, alpha)
-            elif not alpha:
-                data.sort(key=self._strtod_key_func)
-            else:
-                data.sort()
-            if desc:
-                data = data[::-1]
-            if not (start is None and num is None):
-                data = data[start:start + num]
-
-            if get:
-                data = self._retrive_data_from_sort(data, get)
-
-            if store is not None:
-                if data_type == b("set"):
-                    self.delete(store)
-                    self.rpush(store, *data)
-                elif data_type == b("list"):
-                    self.delete(store)
-                    self.rpush(store, *data)
-                else:
-                    raise RedisClusterException("Unable to store sorted data for data type : {0}".format(data_type))
-
-                return len(data)
-
-            if groups:
-                if not get or isinstance(get, basestring) or len(get) < 2:
-                    raise DataError('when using "groups" the "get" argument '
-                                    'must be specified and contain at least '
-                                    'two keys')
-                n = len(get)
-                return list(izip(*[data[i::n] for i in range(n)]))
-            else:
-                return data
-        except KeyError:
-            return []
-
-    def _retrive_data_from_sort(self, data, get):
-        """
-        Used by sort()
-        """
-        if get is not None:
-            if isinstance(get, basestring):
-                get = [get]
-            new_data = []
-            for k in data:
-                for g in get:
-                    single_item = self._get_single_item(k, g)
-                    new_data.append(single_item)
-            data = new_data
-        return data
-
-    def _get_single_item(self, k, g):
-        """
-        Used by sort()
-        """
-        if getattr(k, "decode", None):
-            k = k.decode("utf-8")
-
-        if '*' in g:
-            g = g.replace('*', k)
-            if '->' in g:
-                key, hash_key = g.split('->')
-                single_item = self.get(key, {}).get(hash_key)
-            else:
-                single_item = self.get(g)
-        elif '#' in g:
-            single_item = k
-        else:
-            single_item = None
-        return b(single_item)
-
-    def _strtod_key_func(self, arg):
-        """
-        Used by sort()
-        """
-        return float(arg)
-
-    def _sort_using_by_arg(self, data, by, alpha):
-        """
-        Used by sort()
-        """
-        if getattr(by, "decode", None):
-            by = by.decode("utf-8")
-
-        def _by_key(arg):
-            if getattr(arg, "decode", None):
-                arg = arg.decode("utf-8")
-
-            key = by.replace('*', arg)
-            if '->' in by:
-                key, hash_key = key.split('->')
-                v = self.hget(key, hash_key)
-                if alpha:
-                    return v
-                else:
-                    return float(v)
-            else:
-                return self.get(key)
-        data.sort(key=_by_key)
-
     ###
     # Set commands
 
@@ -1117,91 +1100,4 @@ class StrictRedisCluster(StrictRedis):
         return ''.join(random.choice(chars) for _ in range(size))
 
 
-class RedisCluster(StrictRedisCluster):
-    """
-    Provides backwards compatibility with older versions of redis-py that
-    changed arguments to some commands to be more Pythonic, sane, or by
-    accident.
-    """
-    # Overridden callbacks
-    RESPONSE_CALLBACKS = dict_merge(
-        StrictRedis.RESPONSE_CALLBACKS,
-        {
-            'TTL': lambda r: r >= 0 and r or None,
-            'PTTL': lambda r: r >= 0 and r or None,
-        }
-    )
-
-    def pipeline(self, transaction=True, shard_hint=None):
-        """
-        Return a new pipeline object that can queue multiple commands for
-        later execution. ``transaction`` indicates whether all commands
-        should be executed atomically. Apart from making a group of operations
-        atomic, pipelines are useful for reducing the back-and-forth overhead
-        between the client and server.
-        """
-        if shard_hint:
-            raise RedisClusterException("shard_hint is deprecated in cluster mode")
-
-        if transaction:
-            raise RedisClusterException("transaction is deprecated in cluster mode")
-
-        return StrictClusterPipeline(
-            connection_pool=self.connection_pool,
-            startup_nodes=self.connection_pool.nodes.startup_nodes,
-            response_callbacks=self.response_callbacks
-        )
-
-    def setex(self, name, value, time):
-        """
-        Set the value of key ``name`` to ``value`` that expires in ``time``
-        seconds. ``time`` can be represented by an integer or a Python
-        timedelta object.
-        """
-        if isinstance(time, datetime.timedelta):
-            time = time.seconds + time.days * 24 * 3600
-
-        return self.execute_command('SETEX', name, time, value)
-
-    def lrem(self, name, value, num=0):
-        """
-        Remove the first ``num`` occurrences of elements equal to ``value``
-        from the list stored at ``name``.
-        The ``num`` argument influences the operation in the following ways:
-            num > 0: Remove elements equal to value moving from head to tail.
-            num < 0: Remove elements equal to value moving from tail to head.
-            num = 0: Remove all elements equal to value.
-        """
-        return self.execute_command('LREM', name, num, value)
-
-    def zadd(self, name, *args, **kwargs):
-        """
-        NOTE: The order of arguments differs from that of the official ZADD
-        command. For backwards compatability, this method accepts arguments
-        in the form of name1, score1, name2, score2, while the official Redis
-        documents expects score1, name1, score2, name2.
-        If you're looking to use the standard syntax, consider using the
-        StrictRedis class. See the API Reference section of the docs for more
-        information.
-        Set any number of element-name, score pairs to the key ``name``. Pairs
-        can be specified in two ways:
-        As *args, in the form of: name1, score1, name2, score2, ...
-        or as **kwargs, in the form of: name1=score1, name2=score2, ...
-        The following example would add four values to the 'my-key' key:
-        redis.zadd('my-key', 'name1', 1.1, 'name2', 2.2, name3=3.3, name4=4.4)
-        """
-        pieces = []
-
-        if args:
-            if len(args) % 2 != 0:
-                raise RedisError("ZADD requires an equal number of values and scores")
-            pieces.extend(reversed(args))
-
-        for pair in iteritems(kwargs):
-            pieces.append(pair[1])
-            pieces.append(pair[0])
-
-        return self.execute_command('ZADD', name, *pieces)
-
-
-from rediscluster.pipeline import StrictClusterPipeline
+from rediscluster.pipeline import ClusterPipeline
