@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # python std lib
+from __future__ import unicode_literals
 import os
 import random
 import threading
@@ -12,12 +13,13 @@ from .nodemanager import NodeManager
 from .exceptions import (
     RedisClusterException, AskError, MovedError,
     TryAgainError, ClusterDownError, ClusterCrossSlotError,
+    MasterDownError,
 )
 
 # 3rd party imports
 from redis._compat import nativestr
 from redis.client import dict_merge
-from redis.connection import ConnectionPool, Connection, DefaultParser
+from redis.connection import ConnectionPool, Connection, DefaultParser, SSLConnection
 from redis.exceptions import ConnectionError
 
 
@@ -31,6 +33,7 @@ class ClusterParser(DefaultParser):
             'MOVED': MovedError,
             'CLUSTERDOWN': ClusterDownError,
             'CROSSSLOT': ClusterCrossSlotError,
+            'MASTERDOWN': MasterDownError,
         })
 
 
@@ -57,6 +60,34 @@ class ClusterConnection(Connection):
                 raise ConnectionError('READONLY command failed')
 
 
+class SSLClusterConnection(SSLConnection):
+    """
+    Manages TCP communication over TLS/SSL to and from a Redis cluster
+    Usage:
+        pool = ClusterConnectionPool(connection_class=SSLClusterConnection, ...)
+        client = RedisCluster(connection_pool=pool)
+    """
+    description_format = "SSLClusterConnection<host=%(host)s,port=%(port)s,db=%(db)s>"
+
+    def __init__(self, **kwargs):
+        self.readonly = kwargs.pop('readonly', False)
+        kwargs['parser_class'] = ClusterParser
+        super(SSLClusterConnection, self).__init__(**kwargs)
+
+    def on_connect(self):
+        '''
+        Initialize the connection, authenticate and select a database and send READONLY if it is
+        set during object initialization.
+        '''
+        super(SSLClusterConnection, self).on_connect()
+
+        if self.readonly:
+            self.send_command('READONLY')
+
+            if nativestr(self.read_response()) != 'OK':
+                raise ConnectionError('READONLY command failed')
+
+
 class UnixDomainSocketConnection(Connection):
     """
     """
@@ -69,14 +100,20 @@ class ClusterConnectionPool(ConnectionPool):
     """
     RedisClusterDefaultTimeout = None
 
-    def __init__(self, startup_nodes=None, init_slot_cache=True, connection_class=ClusterConnection,
+    def __init__(self, startup_nodes=None, init_slot_cache=True, connection_class=None,
                  max_connections=None, max_connections_per_node=False, reinitialize_steps=None,
-                 skip_full_coverage_check=False, **connection_kwargs):
+                 skip_full_coverage_check=False, nodemanager_follow_cluster=False, **connection_kwargs):
         """
         :skip_full_coverage_check:
             Skips the check of cluster-require-full-coverage config, useful for clusters
             without the CONFIG command (like aws)
+        :nodemanager_follow_cluster:
+            The node manager will during initialization try the last set of nodes that
+            it was operating on. This will allow the client to drift along side the cluster
+            if the cluster nodes move around alot.
         """
+        if connection_class is None:
+            connection_class = ClusterConnection
         super(ClusterConnectionPool, self).__init__(connection_class=connection_class, max_connections=max_connections)
 
         # Special case to make from_url method compliant with cluster setting.
@@ -92,8 +129,18 @@ class ClusterConnectionPool(ConnectionPool):
         self.max_connections = max_connections or 2 ** 31
         self.max_connections_per_node = max_connections_per_node
 
-        self.nodes = NodeManager(startup_nodes, reinitialize_steps=reinitialize_steps,
-                                 skip_full_coverage_check=skip_full_coverage_check, **connection_kwargs)
+        if connection_class == SSLClusterConnection:
+            connection_kwargs['ssl'] = True  # needed in Redis init
+
+        self.nodes = NodeManager(
+            startup_nodes,
+            reinitialize_steps=reinitialize_steps,
+            skip_full_coverage_check=skip_full_coverage_check,
+            max_connections=self.max_connections,
+            nodemanager_follow_cluster=nodemanager_follow_cluster,
+            **connection_kwargs
+        )
+
         if init_slot_cache:
             self.nodes.initialize()
 
@@ -244,7 +291,7 @@ class ClusterConnectionPool(ConnectionPool):
 
         raise Exception("Cant reach a single startup node.")
 
-    def get_connection_by_key(self, key):
+    def get_connection_by_key(self, key, command):
         """
         """
         if not key:
@@ -260,7 +307,7 @@ class ClusterConnectionPool(ConnectionPool):
 
         try:
             return self.get_connection_by_node(self.get_node_by_slot(slot))
-        except KeyError:
+        except (KeyError, RedisClusterException) as exc:
             return self.get_random_connection()
 
     def get_connection_by_node(self, node):
@@ -283,9 +330,14 @@ class ClusterConnectionPool(ConnectionPool):
     def get_master_node_by_slot(self, slot):
         """
         """
-        return self.nodes.slots[slot][0]
+        try:
+            return self.nodes.slots[slot][0]
+        except KeyError as ke:
+            raise RedisClusterException('Slot "{slot}" not covered by the cluster. "skip_full_coverage_check={skip_full_coverage_check}"'.format(
+                slot=slot, skip_full_coverage_check=self.nodes._skip_full_coverage_check,
+            ))
 
-    def get_node_by_slot(self, slot):
+    def get_node_by_slot(self, slot, *args, **kwargs):
         """
         """
         return self.get_master_node_by_slot(slot)
@@ -296,24 +348,79 @@ class ClusterReadOnlyConnectionPool(ClusterConnectionPool):
     Readonly connection pool for rediscluster
     """
 
-    def __init__(self, startup_nodes=None, init_slot_cache=True, connection_class=ClusterConnection,
-                 max_connections=None, **connection_kwargs):
+    def __init__(self, startup_nodes=None, init_slot_cache=True, connection_class=None,
+                 max_connections=None, nodemanager_follow_cluster=False, **connection_kwargs):
         """
         """
+        if connection_class is None:
+            connection_class = ClusterConnection
         super(ClusterReadOnlyConnectionPool, self).__init__(
             startup_nodes=startup_nodes,
             init_slot_cache=init_slot_cache,
             connection_class=connection_class,
             max_connections=max_connections,
             readonly=True,
+            nodemanager_follow_cluster=nodemanager_follow_cluster,
             **connection_kwargs)
 
-    def get_node_by_slot(self, slot):
+        self.master_node_commands = ('SCAN', 'SSCAN', 'HSCAN', 'ZSCAN')
+
+    def get_connection_by_key(self, key, command):
         """
         """
+        if not key:
+            raise RedisClusterException("No way to dispatch this command to Redis Cluster.")
+
+        if command in self.master_node_commands:
+            return self.get_master_connection_by_slot(self.nodes.keyslot(key))
+        else:
+            return self.get_random_master_slave_connection_by_slot(self.nodes.keyslot(key))
+
+    def get_master_connection_by_slot(self, slot):
+        """
+        Returns a connection for the Master node for the specefied slot.
+
+        Do not return a random node if master node is not available for any reason.
+        """
+        self._checkpid()
+        return self.get_connection_by_node(self.get_node_by_slot(slot))
+
+    def get_random_master_slave_connection_by_slot(self, slot):
+        """
+        Returns a random connection from the set of (master + slaves) for the
+        specefied slot. If connection is not reachable then return a random connection.
+        """
+        self._checkpid()
+
+        try:
+            return self.get_node_by_slot_random(self.get_node_by_slot(slot))
+        except KeyError:
+            return self.get_random_connection()
+
+    def get_node_by_slot_random(self, slot):
+        """
+        Return a random node for the specified slot.
+        """
         return random.choice(self.nodes.slots[slot])
 
 
+class ClusterWithReadReplicasConnectionPool(ClusterConnectionPool):
+    """
+    Custom connection pool for rediscluster with load balancing across read replicas
+    """
+
+    def get_node_by_slot(self, slot, read_command=False):
+        """
+        Get a random node from the slot, including master
+        """
+        nodes_in_slot = self.nodes.slots[slot]
+        if read_command:
+            random_index = random.randrange(0, len(nodes_in_slot))
+            return nodes_in_slot[random_index]
+        else:
+            return nodes_in_slot[0]
+
+
 @contextmanager
 def by_node_context(pool, node):
     """
