@@ -2,16 +2,16 @@
 
 # python std lib
 import random
-import sys
 
 # rediscluster imports
 from .crc import crc16
 from .exceptions import RedisClusterException
 
 # 3rd party imports
-from redis import StrictRedis
-from redis._compat import unicode
-from redis import ConnectionError
+from redis import Redis
+from redis._compat import unicode, bytes, long, basestring
+from redis.connection import Encoder
+from redis import ConnectionError, TimeoutError, ResponseError
 
 
 class NodeManager(object):
@@ -19,11 +19,15 @@ class NodeManager(object):
     """
     RedisClusterHashSlots = 16384
 
-    def __init__(self, startup_nodes=None, reinitialize_steps=None, skip_full_coverage_check=False, **connection_kwargs):
+    def __init__(self, startup_nodes=None, reinitialize_steps=None, skip_full_coverage_check=False, nodemanager_follow_cluster=False, **connection_kwargs):
         """
         :skip_full_coverage_check:
             Skips the check of cluster-require-full-coverage config, useful for clusters
             without the CONFIG command (like aws)
+        :nodemanager_follow_cluster:
+            The node manager will during initialization try the last set of nodes that
+            it was operating on. This will allow the client to drift along side the cluster
+            if the cluster nodes move around alot.
         """
         self.connection_kwargs = connection_kwargs
         self.nodes = {}
@@ -33,49 +37,27 @@ class NodeManager(object):
         self.reinitialize_counter = 0
         self.reinitialize_steps = reinitialize_steps or 25
         self._skip_full_coverage_check = skip_full_coverage_check
+        self.nodemanager_follow_cluster = nodemanager_follow_cluster
+        self.encoder = Encoder(
+            connection_kwargs.get('encoding', 'utf-8'),
+            connection_kwargs.get('encoding_errors', 'strict'),
+            connection_kwargs.get('decode_responses', False)
+        )
 
         if not self.startup_nodes:
             raise RedisClusterException("No startup nodes provided")
 
-        # Minor performance tweak to avoid having to check inside the method
-        # for each call to keyslot method.
-        if sys.version_info[0] < 3:
-            self.keyslot = self.keyslot_py_2
-        else:
-            self.keyslot = self.keyslot_py_3
-
-    def keyslot_py_2(self, key):
+    def keyslot(self, key):
         """
         Calculate keyslot for a given key.
         Tuned for compatibility with python 2.7.x
         """
-        k = unicode(key)
+        k = self.encoder.encode(key)
 
-        start = k.find("{")
+        start = k.find(b"{")
 
         if start > -1:
-            end = k.find("}", start + 1)
-            if end > -1 and end != start + 1:
-                k = k[start + 1:end]
-
-        return crc16(k) % self.RedisClusterHashSlots
-
-    def keyslot_py_3(self, key):
-        """
-        Calculate keyslot for a given key.
-        Tuned for compatibility with supported python 3.x versions
-        """
-        try:
-            # Handle bytes case
-            k = str(key, encoding='utf-8')
-        except TypeError:
-            # Convert others to str.
-            k = str(key)
-
-        start = k.find("{")
-
-        if start > -1:
-            end = k.find("}", start + 1)
+            end = k.find(b"}", start + 1)
             if end > -1 and end != start + 1:
                 k = k[start + 1:end]
 
@@ -155,15 +137,11 @@ class NodeManager(object):
             'decode_responses',
         )
         connection_kwargs = {k: v for k, v in self.connection_kwargs.items() if k in set(allowed_keys) - set(disabled_keys)}
-        return StrictRedis(host=host, port=port, decode_responses=decode_responses, **connection_kwargs)
+        return Redis(host=host, port=port, decode_responses=decode_responses, **connection_kwargs)
 
     def initialize(self):
         """
         Init the slots cache by asking all startup nodes what the current cluster configuration is
-
-        TODO: Currently the last node will have the last say about how the configuration is setup.
-        Maybe it should stop to try after it have correctly covered all slots or when one node is reached
-         and it could execute CLUSTER SLOTS command.
         """
         nodes_cache = {}
         tmp_slots = {}
@@ -172,13 +150,26 @@ class NodeManager(object):
         disagreements = []
         startup_nodes_reachable = False
 
-        for node in self.orig_startup_nodes:
+        nodes = self.orig_startup_nodes
+
+        # With this option the client will attempt to connect to any of the previous set of nodes instead of the original set of nodes
+        if self.nodemanager_follow_cluster:
+            nodes = self.startup_nodes
+
+        for node in nodes:
             try:
                 r = self.get_redis_link(host=node["host"], port=node["port"], decode_responses=True)
                 cluster_slots = r.execute_command("cluster", "slots")
                 startup_nodes_reachable = True
-            except ConnectionError:
+            except (ConnectionError, TimeoutError):
                 continue
+            except ResponseError as e:
+                # Isn't a cluster connection, so it won't parse these exceptions automatically
+                message = e.__str__()
+                if 'CLUSTERDOWN' in message or 'MASTERDOWN' in message:
+                    continue
+                else:
+                    raise RedisClusterException("ERROR sending 'cluster slots' command to redis server: {0}".format(node))
             except Exception:
                 raise RedisClusterException("ERROR sending 'cluster slots' command to redis server: {0}".format(node))
 
@@ -189,7 +180,7 @@ class NodeManager(object):
             if (len(cluster_slots) == 1 and len(cluster_slots[0][2][0]) == 0 and len(self.startup_nodes) == 1):
                 cluster_slots[0][2][0] = self.startup_nodes[0]['host']
 
-            # No need to decode response because StrictRedis should handle that for us...
+            # No need to decode response because Redis should handle that for us...
             for slot in cluster_slots:
                 master_node = slot[2]
 
@@ -263,8 +254,13 @@ class NodeManager(object):
         nodes = nodes_cache or self.nodes
 
         def node_require_full_coverage(node):
-            r_node = self.get_redis_link(host=node["host"], port=node["port"], decode_responses=True)
-            return "yes" in r_node.config_get("cluster-require-full-coverage").values()
+            try:
+                r_node = self.get_redis_link(host=node["host"], port=node["port"], decode_responses=True)
+                return "yes" in r_node.config_get("cluster-require-full-coverage").values()
+            except ConnectionError:
+                return False
+            except Exception:
+                raise RedisClusterException("ERROR sending 'config get cluster-require-full-coverage' command to redis server: {0}".format(node))
 
         # at least one node should have cluster-require-full-coverage yes
         return any(node_require_full_coverage(node) for node in nodes.values())
