@@ -126,7 +126,7 @@ def parse_cluster_slots(resp, **options):
     current_host = options.get('current_host', '')
 
     def fix_server(*args):
-        return (args[0] or current_host, args[1])
+        return (nativestr(args[0]) or current_host, args[1])
 
     slots = {}
     for slot in resp:
@@ -145,6 +145,7 @@ def parse_cluster_nodes(resp, **options):
     @see: http://redis.io/commands/cluster-nodes  # string
     @see: http://redis.io/commands/cluster-slaves # list of string
     """
+    resp = nativestr(resp)
     current_host = options.get('current_host', '')
 
     def parse_slots(s):
@@ -181,12 +182,14 @@ def parse_cluster_nodes(resp, **options):
         self_id, addr, flags, master_id, ping_sent, \
             pong_recv, config_epoch, link_state = parts[:8]
 
-        host, port = addr.rsplit(':', 1)
+        host, ports = addr.rsplit(':', 1)
+        port, _, cluster_port = ports.partition('@')
 
         node = {
             'id': self_id,
             'host': host or current_host,
             'port': int(port),
+            'cluster-bus-port': int(cluster_port) if cluster_port else 10000 + int(port),
             'flags': tuple(flags.split(',')),
             'master': master_id if master_id != '-' else None,
             'ping-sent': int(ping_sent),
@@ -203,3 +206,52 @@ def parse_cluster_nodes(resp, **options):
         nodes.append(node)
 
     return nodes
+
+
+def parse_pubsub_channels(command, res, **options):
+    """
+    Result callback, handles different return types
+    switchable by the `aggregate` flag.
+    """
+    aggregate = options.get('aggregate', True)
+    if not aggregate:
+        return res
+    return merge_result(command, res)
+
+
+def parse_pubsub_numpat(command, res, **options):
+    """
+    Result callback, handles different return types
+    switchable by the `aggregate` flag.
+    """
+    aggregate = options.get('aggregate', True)
+    if not aggregate:
+        return res
+
+    numpat = 0
+    for node, node_numpat in res.items():
+        numpat += node_numpat
+    return numpat
+
+
+def parse_pubsub_numsub(command, res, **options):
+    """
+    Result callback, handles different return types
+    switchable by the `aggregate` flag.
+    """
+    aggregate = options.get('aggregate', True)
+    if not aggregate:
+        return res
+
+    numsub_d = dict()
+    for _, numsub_tups in res.items():
+        for channel, numsubbed in numsub_tups:
+            try:
+                numsub_d[channel] += numsubbed
+            except KeyError:
+                numsub_d[channel] = numsubbed
+
+    ret_numsub = []
+    for channel, numsub in numsub_d.items():
+        ret_numsub.append((channel, numsub))
+    return ret_numsub
