@@ -1,21 +1,23 @@
 # -*- coding: utf-8 -*-
 
 # python std lib
-from __future__ import with_statement
+from __future__ import unicode_literals
 import threading
 import time
 
 # rediscluster imports
-from rediscluster.client import StrictRedisCluster
+from rediscluster.client import RedisCluster
 
 # 3rd party imports
 import pytest
 
 # import redis
-from redis import StrictRedis, Redis
+from redis import Redis
 from redis.exceptions import ConnectionError
-from redis._compat import basestring, u, unichr
+from redis._compat import basestring, unichr
 
+from .conftest import _get_client
+from .conftest import skip_if_server_version_lt, skip_if_redis_py_version_lt
 
 def wait_for_message(pubsub, timeout=0.5, ignore_subscribe_messages=False):
     now = time.time()
@@ -34,7 +36,7 @@ def make_message(type, channel, data, pattern=None):
     return {
         'type': type,
         'pattern': pattern and pattern.encode('utf-8') or None,
-        'channel': channel.encode('utf-8'),
+        'channel': channel and channel.encode('utf-8') or None,
         'data': data.encode('utf-8') if isinstance(data, basestring) else data
     }
 
@@ -47,7 +49,7 @@ def make_subscribe_test_data(pubsub, type):
             'unsub_type': 'unsubscribe',
             'sub_func': pubsub.subscribe,
             'unsub_func': pubsub.unsubscribe,
-            'keys': ['foo', 'bar', u('uni') + unichr(4456) + u('code')]
+            'keys': ['foo', 'bar', 'uni' + unichr(4456) + 'code']
         }
     elif type == 'pattern':
         return {
@@ -56,7 +58,7 @@ def make_subscribe_test_data(pubsub, type):
             'unsub_type': 'punsubscribe',
             'sub_func': pubsub.psubscribe,
             'unsub_func': pubsub.punsubscribe,
-            'keys': ['f*', 'b*', u('uni') + unichr(4456) + u('*')]
+            'keys': ['f*', 'b*', 'uni' + unichr(4456) + '*']
         }
     assert False, 'invalid subscribe type: {0}'.format(type)
 
@@ -220,12 +222,12 @@ class TestPubSubMessages(object):
     Bug: Currently in cluster mode publish command will behave different then in
          standard/non cluster mode. See (docs/Pubsub.md) for details.
 
-         Currently StrictRedis instances will be used to test pubsub because they
+         Currently Redis instances will be used to test pubsub because they
          are easier to work with.
     """
 
     def get_strict_redis_node(self, port, host="127.0.0.1"):
-        return StrictRedis(port=port, host=host)
+        return Redis(port=port, host=host)
 
     def setup_method(self, *args):
         self.message = None
@@ -308,7 +310,7 @@ class TestPubSubMessages(object):
     @pytest.mark.xfail(reason="Pattern pubsub do not work currently")
     def test_unicode_channel_message_handler(self, r):
         p = r.pubsub(ignore_subscribe_messages=True)
-        channel = u('uni') + unichr(4456) + u('code')
+        channel = 'uni' + unichr(4456) + 'code'
         channels = {channel: self.message_handler}
         print(channels)
         p.subscribe(**channels)
@@ -319,8 +321,8 @@ class TestPubSubMessages(object):
     @pytest.mark.xfail(reason="Pattern pubsub do not work currently")
     def test_unicode_pattern_message_handler(self, r):
         p = r.pubsub(ignore_subscribe_messages=True)
-        pattern = u('uni') + unichr(4456) + u('*')
-        channel = u('uni') + unichr(4456) + u('code')
+        pattern = 'uni' + unichr(4456) + '*'
+        channel = 'uni' + unichr(4456) + 'code'
         p.psubscribe(**{pattern: self.message_handler})
         assert r.publish(channel, 'test message') == 1
         assert wait_for_message(p) is None
@@ -331,9 +333,9 @@ class TestPubSubMessages(object):
 class TestPubSubAutoDecoding(object):
     "These tests only validate that we get unicode values back"
 
-    channel = u('uni') + unichr(4456) + u('code')
-    pattern = u('uni') + unichr(4456) + u('*')
-    data = u('abc') + unichr(4458) + u('123')
+    channel = 'uni' + unichr(4456) + 'code'
+    pattern = 'uni' + unichr(4456) + '*'
+    data = 'abc' + unichr(4458) + '123'
 
     def make_message(self, type, channel, data, pattern=None):
         return {
@@ -399,7 +401,7 @@ class TestPubSubAutoDecoding(object):
         # test that we reconnected to the correct channel
         p.connection.disconnect()
         assert wait_for_message(p) is None  # should reconnect
-        new_data = self.data + u('new data')
+        new_data = self.data + 'new data'
         o.publish(self.channel, new_data)
         assert wait_for_message(p) is None
         assert self.message == self.make_message('message', self.channel,
@@ -418,7 +420,7 @@ class TestPubSubAutoDecoding(object):
         # test that we reconnected to the correct pattern
         p.connection.disconnect()
         assert wait_for_message(p) is None  # should reconnect
-        new_data = self.data + u('new data')
+        new_data = self.data + 'new data'
         o.publish(self.channel, new_data)
         assert wait_for_message(p) is None
         assert self.message == self.make_message('pmessage', self.channel,
@@ -443,7 +445,7 @@ def test_pubsub_thread_publish():
     """
     startup_nodes = [{"host": "127.0.0.1", "port": "7000"}]
 
-    r = StrictRedisCluster(
+    r = RedisCluster(
         startup_nodes=startup_nodes,
         decode_responses=True,
         max_connections=16,
@@ -474,3 +476,57 @@ def test_pubsub_thread_publish():
             t.start()
     except Exception:
         print("Error: unable to start thread")
+
+
+class TestPubSubPubSubSubcommands(object):
+
+    @skip_if_server_version_lt('2.8.0')
+    @pytest.mark.xfail(reason="Pubsub is not fully supported in cluster mode")
+    def test_pubsub_channels(self, r):
+        p = r.pubsub(ignore_subscribe_messages=True)
+        p.subscribe('foo', 'bar', 'baz', 'quux')
+        channels = sorted(r.pubsub_channels())
+        assert channels == [b'bar', b'baz', b'foo', b'quux']
+
+    @skip_if_server_version_lt('2.8.0')
+    @pytest.mark.xfail(reason="Pubsub is not fully supported in cluster mode")
+    def test_pubsub_numsub(self, r):
+        p1 = r.pubsub(ignore_subscribe_messages=True)
+        p1.subscribe('foo', 'bar', 'baz')
+        p2 = r.pubsub(ignore_subscribe_messages=True)
+        p2.subscribe('bar', 'baz')
+        p3 = r.pubsub(ignore_subscribe_messages=True)
+        p3.subscribe('baz')
+
+        channels = [(b'foo', 1), (b'bar', 2), (b'baz', 3)]
+        assert channels == r.pubsub_numsub('foo', 'bar', 'baz')
+
+    @skip_if_server_version_lt('2.8.0')
+    @pytest.mark.xfail(reason="Pubsub is not fully supported in cluster mode")
+    def test_pubsub_numpat(self, r):
+        p = r.pubsub(ignore_subscribe_messages=True)
+        p.psubscribe('*oo', '*ar', 'b*z')
+        assert r.pubsub_numpat() == 3
+
+
+class TestPubSubPings(object):
+
+    @skip_if_server_version_lt('3.0.0')
+    @pytest.mark.xfail(reason="Pubsub is not fully supported in cluster mode")
+    def test_send_pubsub_ping(self, r):
+        p = r.pubsub(ignore_subscribe_messages=True)
+        p.subscribe('foo')
+        p.ping()
+        assert wait_for_message(p) == make_message(type='pong', channel=None,
+                                                   data='',
+                                                   pattern=None)
+
+    @skip_if_server_version_lt('3.0.0')
+    @pytest.mark.xfail(reason="Pattern pubsub is not fully supported in cluster mode")
+    def test_send_pubsub_ping_message(self, r):
+        p = r.pubsub(ignore_subscribe_messages=True)
+        p.subscribe('foo')
+        p.ping(message='hello world')
+        assert wait_for_message(p) == make_message(type='pong', channel=None,
+                                                   data='hello world',
+                                                   pattern=None)
